// -*- C++ -*-
/*!
 * @file  InEnvInfo.cpp
 * @brief EnvInfo Input Component
 * @date $Date$
 *
 * $Id$
 */

#include "InEnvInfo.h"

SocketManager m_socket;
std::string recvStr;
bool recvFlag;
void recvData(void *arg);

// Module specification
// <rtc-template block="module_spec">
static const char* inenvinfo_spec[] =
  {
    "implementation_id", "InEnvInfo",
    "type_name",         "InEnvInfo",
    "description",       "EnvInfo Input Component",
    "version",           "1.0.0",
    "vendor",            "VenderName",
    "category",          "Category",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.IPAddress", "127.0.0.1",
    "conf.default.Port", "7003",
    // Widget
    "conf.__widget__.IPAddress", "text",
    "conf.__widget__.Port", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
InEnvInfo::InEnvInfo(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_outPositionDataOut("outPositionData", m_outPositionData)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
InEnvInfo::~InEnvInfo()
{
}



RTC::ReturnCode_t InEnvInfo::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  addOutPort("outPositionData", m_outPositionDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("IPAddress", m_IPAddress, "127.0.0.1");
  bindParameter("Port", m_Port, "7003");
  
  // </rtc-template>
  return RTC::RTC_OK;
}


RTC::ReturnCode_t InEnvInfo::onFinalize()
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t InEnvInfo::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InEnvInfo::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t InEnvInfo::onActivated(RTC::UniqueId ec_id)
{
	m_connect = m_socket.FirstConnect(m_IPAddress, (unsigned short)m_Port);
	recvFlag = false;
	m_hThread = (HANDLE)_beginthread(recvData, 0, NULL);
	std::string request;
    request = "\n";
	m_socket.SendRequest(request);
	return RTC::RTC_OK;
}


RTC::ReturnCode_t InEnvInfo::onDeactivated(RTC::UniqueId ec_id)
{
	WaitForSingleObject(m_hThread,INFINITE);
	CloseHandle(m_hThread);
    std::string request = "Q";
	m_socket.SendRequest(request);
	m_socket.CuttingConnect();
	return RTC::RTC_OK;
}


RTC::ReturnCode_t InEnvInfo::onExecute(RTC::UniqueId ec_id)
{
	if (m_connect && recvFlag == true) {
		if(recvStr.length() > 0){
			std::cout << "LOG OUTPUT START" << std::endl;
			m_outPositionData.data = CORBA::string_dup(recvStr.data());
			std::cout << m_outPositionData.data;
			m_outPositionDataOut.write();
			std::cout << "LOG OUTPUT END" << std::endl << std::endl;
		}
		recvFlag = false;
		CloseHandle(m_hThread);
		m_hThread = (HANDLE)_beginthread(&recvData, 0, NULL);
		std::string request;
    		request = "\n";
		std::cout << "request: <LF>" << std::endl;		
		m_socket.SendRequest(request);
	}
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t InEnvInfo::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InEnvInfo::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InEnvInfo::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InEnvInfo::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t InEnvInfo::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

void recvData(void *arg)
{
	recvStr = m_socket.ReceiveData();
	recvFlag = true;
}

extern "C"
{
 
  void InEnvInfoInit(RTC::Manager* manager)
  {
    coil::Properties profile(inenvinfo_spec);
    manager->registerFactory(profile,
                             RTC::Create<InEnvInfo>,
                             RTC::Delete<InEnvInfo>);
  }
  
};


